package gonumops

import (
	"flow"
	"flow/registry"
	"math"
	"math/rand"

	"gonum.org/v1/gonum/mat"
)

// Tensor wrapper
type Tensor interface {
	mat.Matrix
}

// New registry
func New() *registry.R {

	r := registry.New()

	r.Add(
		myVar,
		normFloat,
		tensorNew,
		tensorMul,
		tensorTranspose,
		logistic,
	).Description("gonum functions").
		Tags("gonum").
		Extra("style", registry.M{"color": "#953"})

	return r
}

func myVar(f *flow.Flow, placeholder string, init flow.Data) flow.Data {

	if _, ok := f.Data[placeholder]; !ok {
		f.Data[placeholder] = init
	}
	return f.Data[placeholder]
}

func normFloat(n int) []float64 {
	data := make([]float64, n)
	for i := range data {
		data[i] = rand.NormFloat64()
	}
	return data
}

func tensorNew(r, c int, data []float64) Tensor {
	return mat.NewDense(r, c, data)
}

func tensorMul(a Tensor, b Tensor) Tensor {
	var c mat.Dense
	c.Mul(a, b)
	return &c
}

func tensorTranspose(a Tensor) Tensor {
	return a.T()
}

func logistic(a Tensor) Tensor {
	v := a.(mat.Vector)
	ret := mat.NewVecDense(v.Len(), nil)
	// Should be a vector perhaps
	for i := 0; i < v.Len(); i++ {
		ret.SetVec(i, activator(v.AtVec(i)))
	}
	return ret
}

func activator(v float64) float64 {
	return 1 / (1 + math.Exp(-v))
}
